from ..dios import *
from .test_setup import *
import pytest

s1 = pd.Series(range(10), index=range(10))
s2 = pd.Series(range(5, 10), index=range(5, 10))
s3 = pd.Series(range(1, 30, 2), index=range(1, 30, 2))
s4 = pd.Series(np.linspace(7, 13, 9), index=range(3, 12))
s1.name, s2.name, s3.name, s4.name = 'a', 'b', 'c', 'd'
d1 = DictOfSeries(data=dict(a=s1.copy(), b=s2.copy(), c=s3.copy(), d=s4.copy()))

blist = [True, False, False, True]
b = pd.Series([True, False] * 5, index=[1, 2, 3, 4, 5] + [6, 8, 10, 12, 14])
B = d1 > 5


@pytest.mark.parametrize(('idxer', 'exp'), [(slice(None), [s1 == s1, s2 == s2, s3 == s3, s4 == s4]),
                                            (B, [s1 > 5, s2 > 5, s3 > 5, s4 > 5])
                                            ])
def test__setitem_single(idxer, exp):
    d = d1.copy()
    d[idxer] = 99
    for i, c in enumerate(d1):
        assert ((d[c] == 99) == exp[i]).all()


@pytest.mark.parametrize(('idxer', 'exp'), [(slice(None), [s1 == s1, s2 == s2, s3 == s3, s4 == s4]),
                                            (blist, [s1 == s1, s2 != s2, s3 != s3, s4 == s4]),
                                            ])
def test__setitem_scalar_loc(idxer, exp):
    d = d1.copy()
    d.loc[:, idxer] = 99
    for i, c in enumerate(d1):
        assert ((d[c] == 99) == exp[i]).all()


BLIST = [True, False, False, True]

LISTIDXER = [['a'], ['a', 'c'], pd.Series(['a', 'c'])]
BOOLIDXER = [pd.Series(BLIST), d1.copy() > 10]
SLICEIDXER = [slice(None), slice(-3, -1), slice(-1, 3), slice(None, None, 3)]
MULTIIDXER = []  # [d1 > 9, d1 != d1, d1 == d1]
EMPTYIDEXER = [[], pd.Series(), slice(3, 3), slice(3, -1), DictOfSeries()]

INDEXERS = LISTIDXER + BOOLIDXER + SLICEIDXER + MULTIIDXER + EMPTYIDEXER

LOC_L = [slice(None), slice(2, 8), pd.Series(BLIST, index=[1, 4, 8, 9]), ]
LOC_R = [slice(None), slice('a', 'c'), pd.Series(BLIST, index=list("baxd")), BLIST, ] + LISTIDXER

VALS = [99,
        pd.Series(range(4, 10), index=range(4, 10)),
        DictOfSeries(dict(a=[33, 33, 33], b=range(30, 90, 2)))
        ]


@pytest.mark.parametrize('idxerL', LOC_L)
@pytest.mark.parametrize('idxerR', LOC_R)
@pytest.mark.parametrize('val', VALS)
def test__setitem__loc(idxerL, idxerR, val):
    d = d1.copy()
    d.loc[idxerL, idxerR] = val
    assert isinstance(d, DictOfSeries)


FAIL_INDEXERS = [['z'], ['a', 'z'], pd.Series(['a', 'z']), BLIST, DictOfSeries(dict(a=[1, 2, 3]))]


@pytest.mark.parametrize('idxer', FAIL_INDEXERS)
def test__setitem__fail(idxer):
    with pytest.raises((ValueError, KeyError)):
        d1[idxer]
